#!/usr/local/bin/perl -w
#
# NAME: esensitivity
# VERSION: 1.0
#........................................................................
# This script makes sensitivity maps.
#........................................................................
#
# Developer: Ian Stewart		SSC-LUX
#
# TODO:
#******* either remove stupid evselect call from asmooth plus the necessary dependencies, or include these dependencies in esensitivity DEPEND. I have a feeling this may be why it is failing at MPE.
################################################################################

my ($testflag);
my $taskname = "esensitivity";

sub esensitivity {

  use SAS;

  my ($i, $boxsize, $command);

  # Prints all output to stderr:
  select(STDERR);

  #......................................................................
  # Read and check parameters:

#***** specify temp file names (or at a least fname root) on CL!

  my $numBkgMaps = parameterCount('bkgmapsets');
SAS::error('tooFewBkgMapSets', "dummy") if ($numBkgMaps <= 0);
#&quit("Too few bkg maps", 1) if ($numBkgMaps <= 0);

#&quit("Wrong num exp maps", 2)
SAS::error('wrongNumExpMaps', "dummy")
  if ($numBkgMaps != parameterCount('expmapsets'));

#&quit("Wrong num weights", 3)
SAS::error('wrongNumWeights', "dummy")
  if ($numBkgMaps != parameterCount('weights'));

#&quit("Wrong num sens maps", 4)
SAS::error('wrongNumSensMaps', "dummy")
  if ($numBkgMaps != parameterCount('sensmapsets'));

  my @bkgMapSets = ();
  my @expMapSets = ();
  my @weights = ();
  my @sensMapSets = ();
  foreach $i (0 .. $numBkgMaps-1) {
    $bkgMapSets[$i]  = stringParameter('bkgmapsets',  $i);
    $expMapSets[$i]  = stringParameter('expmapsets',  $i);
    $weights[$i]     = realParameter(  'weights',     $i);
    $sensMapSets[$i] = stringParameter('sensmapsets', $i);
  }

  my $likemin         = realParameter('likemin');
  my $boxHalfSize   = intParameter('boxhalfsize');
  my $num_axial_bins  = intParameter('naxial');
  my $num_radial_bins = intParameter('nradial');
#  my $e_fraction      = realParameter('efraction');
  my $psfEnergyStyle  = stringParameter('psfestyle');

  my @psfEnergies = ();
  my @dssImages = ();
  if ($psfEnergyStyle eq 'user') {
#&quit("Wrong num psf energies", 5)
SAS::error('wrongNumPsfEnergies', "dummy")
    if ($numBkgMaps != parameterCount('psfenergies'));

    foreach $i (0 .. $numBkgMaps-1) {
      $psfEnergies[$i] = realParameter('psfenergies',  $i);
    }

  } elsif ($psfEnergyStyle eq 'dss') {
#&quit("Wrong num DSS images", 6)
SAS::error('wrongNumDssImages', "dummy")
    if ($numBkgMaps != parameterCount('dssimagesets'));

    foreach $i (0 .. $numBkgMaps-1) {
      $dssImages[$i] = stringParameter('dssimagesets',  $i);
    }

  } else {
SAS::error('badPsfStyle', "dummy");
#&quit("Bad psfestyle.", 7);
  }

  $testflag   = booleanParameter('astest');
  my $cleanup = booleanParameter('cleanup');

  #......................................................................
  # Set up file names and check for existence:

  my $cif = $ENV{'SAS_CCF'};
SAS::error('sasCcfEnvVarNotSet', "dummy") if (!defined($cif));
#&quit("No CIF defined.", 1) if (!defined($cif));
SAS::error('noCifFound', "dummy") if (!-e "$cif");
#&quit("Can't find CIF $cif.", 2) if (!-e "$cif");

  my @boxBkgMapSets = ();
  my @convolverSets = ();
  my @convExpMapSets = ();
  foreach $i (0 .. $numBkgMaps-1) {
    $boxBkgMapSets[$i] = "temp-boxConvBkgMap-$i.fits";
    $convolverSets[$i] = "temp-convolverSet-$i.fits";
    $convExpMapSets[$i] = "temp-convolvedExpMap-$i.fits";
  }

  #......................................................................
  # Main processing:

  if ($testflag) {
#&warn("Script testflag is set. No output "
    SAS::warning('testFlagSet', "Script testflag is set. No output "
      ."will be produced.");
  }

  # . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
  # Make box-convolved background maps:

  foreach $i (0 .. $numBkgMaps-1) {
    if ($boxHalfSize <= 0) {
      system("cp $bkgMapSets[$i] $boxBkgMapSets[$i]");

    } else {
      $boxsize = 2*$boxHalfSize+1;

      $command = "asmooth "
      ."inset=$bkgMapSets[$i] "
      ."outset=$boxBkgMapSets[$i] "
      ."smoothstyle=simple "
      ."convolverstyle=squarebox "
      ."width=$boxsize "
      ."normalize=no "
      ;

SAS::error('asmoothFailed', "dummy") if (&run("$command"));
#&quit("Couldn't box-convolve background map.", 11) if (&run("$command"));
    }
  }

  # . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
  # Make src exposure-in-box maps

  foreach $i (0 .. $numBkgMaps-1) {
    $command = "econvolverprep "
    ."boxhalfsize=$boxHalfSize "
    ."naxial=$num_axial_bins "
    ."nradial=$num_radial_bins "
    ."expmapset=$expMapSets[$i] " # needs it just for wcs
    ."convolversset=$convolverSets[$i] "
    ."psfestyle=$psfEnergyStyle "
    ."efraction=0.9 "
#**** have the task calculate the efraction needed to ensure that the psf array is larger than the box.
    ;

    if ($psfEnergyStyle eq 'user') {
      $command .= "psfenergy=$psfEnergies[$i] ";
    } else {
      $command .= "dssimageset=$dssImages[$i] ";
    }

SAS::error('econvolverprepFailed', "dummy") if (&run("$command"));
#&quit("Couldn't make PSF convolver set.", 12) if (&run("$command"));

    $command = "asmooth "
    ."inset=$expMapSets[$i] "
    ."outset=$convExpMapSets[$i] "
    ."smoothstyle=withset "
    ."inconvolversset=$convolverSets[$i] "
    ."inindeximageset=$convolverSets[$i] "
    ."normalizeset=no "
    ;

SAS::error('asmoothFailed', "dummy") if (&run("$command"));
#&quit("Couldn't psf_convolve exposure map.", 13) if (&run("$command"));
  }

  # . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
  # Run sensitivity:

  $command = "sensitivity -w 5 "
  ."weights='".join(' ', @weights)."' "
  ."boxbkgmapsets='".join(' ', @boxBkgMapSets)."' "
  ."psfexpmapsets='".join(' ', @convExpMapSets)."' "
  ."expmapsets='".join(' ', @expMapSets)."' "
  ."sensmapsets='".join(' ', @sensMapSets)."' "
  ."likemin=$likemin "
  ;

SAS::error('sensitivityFailed', "dummy") if (&run("$command"));
#&quit("Couldn't make raw sensitivity maps.", 14) if (&run("$command"));

  # . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
  # Add together the sensmaps for the relevant bands:

#  foreach $sum_index (keys(%sum_index_hash)) {
#    &add_images($sens_maps_vector[$sum_index], 0
#      , @{$sum_index_hash{$sum_index}});
#  }

  #......................................................................
  # Cleanup:

  if ($cleanup) {
    foreach $i (0 .. $numBkgMaps-1) {
#******
    }
  }

#exit 0;
}

#  #---------------------------------------------------------------------
#  sub add_images {
#  #---------------------------------------------------------------------
#
#    my ($out_set, $temp_set_num, @in_sets) = @_;
#print "     out_set = $out_set\n";
#print "temp_set_num = $temp_set_num\n";
#
#    foreach (@in_sets) {
##print "in_set=$_\n";
#&quit("Can't find $_", 15) if (!$testflag && !-e "$_");
#    }
#
#    if (@in_sets < 1) {
#&quit("Must be at least 1 input to add_images!", 16);
#
#    } elsif(@in_sets == 1) {
#      $command = "mv $in_sets[0] $out_set";
#&quit("Couldn't $command.", 17) if (&run("$command"));
#
#    } elsif(@in_sets == 2) {
#      unlink("$out_set") if (!$testflag && -e "$out_set");
#
#      $command = "farith "
#      ."infil1=$in_sets[0] "
#      ."infil2=$in_sets[1] "
#      ."outfil=$out_set "
#      ."ops=ADD "
#      ;
#
#&quit("Couldn't add images.", 18) if (&run("$command"));
#
#      unlink("$in_sets[0]") if (!$testflag && -e "$in_sets[0]");
#      unlink("$in_sets[1]") if (!$testflag && -e "$in_sets[1]");
#
#    } else {
#      $infile1 = pop @in_sets;
#      $infile2 = pop @in_sets;
#      ++$temp_set_num; 
#      $temp_set = "temp_farith_"."$temp_set_num.ds";
#
#      unlink("$temp_set") if (!$testflag && -e "$temp_set");
#
#      $command = "farith "
#      ."infil1=$infile1 "
#      ."infil2=$infile2 "
#      ."outfil=$temp_set "
#      ."ops=ADD "
#      ;
#
#&quit("Couldn't add images.", 19) if (&run("$command"));
#
#      unlink("$infile1") if (!$testflag && -e "$infile1");
#      unlink("$infile2") if (!$testflag && -e "$infile2");
#
#      push @in_sets, $temp_set;
#      &add_images($out_set, $temp_set_num, @in_sets);
#    }
#
#  }

  #---------------------------------------------------------------------
  sub run {
  #---------------------------------------------------------------------
  # $testflag should be defined globally.

    my ($command) = @_;
    my $status = 0;

    print "\ninvoke $command\n\n";
    if (!$testflag) {
      $status = system($command);
    }

  return $status;
  }

#  #---------------------------------------------------------------------
#  sub warn {
#  #---------------------------------------------------------------------
#
#    foreach (@_) {
#      print "** $taskname: warning, $_\n\n";
#    }
#  }
#
#  #----------------------------------------------------------------------
#  sub quit {
#  #----------------------------------------------------------------------
#  # Expects $taskname to be defined globally.
#
#    my ($message, $status) = @_;
#
#    my $whole_message = "** ";
#    if (defined($taskname)) {
#      $whole_message .= "$taskname: ";
#    }
#    $whole_message .= "ERROR! $message\n\n";
#    print $whole_message;
#
#exit $status;
#  }
